// aflow_chull_jupyter_plotter.cpp automatic generated
std::string AFLOW_CHULL_JUPYTER_PLOTTER_PY="\
# Imports \n\
from aflow_chull import CHull \n\
import json \n\
import re \n\
import numpy as np \n\
from random import sample \n\
import plotly.graph_objs as go \n\
from plotly.offline import init_notebook_mode, iplot \n\
 \n\
 \n\
# Base class for plotting \n\
class Plotter: \n\
    def __init__(self, aflow_path='~/bin/aflow'): \n\
        # Set-up notebook for offline mode \n\
        init_notebook_mode(connected=True) \n\
 \n\
        # Load CHull in case a new json must be created \n\
        self.chull = CHull(aflow_executable=aflow_path) \n\
 \n\
    def plot(self, alloy): \n\
        # seperate alloy into elements \n\
        ems = re.findall('[A-Z][^A-Z]*', alloy) \n\
        em1, em2, em3 = ems[0], ems[1], ems[2] if len(ems) == 3 else None \n\
        # ----- Load CHull json ----- \n\
        hull = self.chull.get_hull(alloy) \n\
        if not em3: \n\
            # 2D Cartesian plot \n\
            _Two_Species_Plotter().plot_CHull(hull, em1, em2) \n\
        else: \n\
            # Ternary plot \n\
            _Three_Species_Plotter().plot_CHull(hull, em1, em2, em3) \n\
         \n\
         \n\
# For plotting 2 species alloys (2D cartesian) \n\
class _Two_Species_Plotter: \n\
    def plot_CHull(self, hull, em1, em2): \n\
        # Set colors, randomly picked from {red, blue, green} \n\
        color_choices = [[1,0,0], [0,1,0], [0,0,1]] \n\
        colors = sample(color_choices, 2) \n\
        left_color, right_color = colors[0], colors[1] \n\
 \n\
        # ----- Create scatter for edges ----- \n\
        # Grab edges \n\
        edges = [dic['vertices_position'] for dic in list(hull['facets_data'].values())[0]] \n\
        # Create scatter for edges \n\
        hull_scatter = self._create_edges_scatter(edges) \n\
 \n\
        # ----- Create scatter for points ----- \n\
        points_scatter = self._create_points_scatter(hull['points_data'], em1, left_color, right_color) \n\
 \n\
        # ----- Plot ----- \n\
        fig = self._create_figure(hull_scatter, points_scatter, em1, em2) \n\
        iplot(fig) \n\
 \n\
    def _create_edges_scatter(self, edges): \n\
        edge_xs = [] \n\
        edge_ys = [] \n\
        for edge in edges: \n\
            point1 = edge[0] \n\
            edge_xs.append(1.0 - point1[0])  # Flip x values because they are reversed in the json output for some reason \n\
            edge_ys.append(point1[1] * 1000.0)  # Multiply by 1000 to match with the points \n\
 \n\
        hull_scatter = go.Scatter( \n\
            x=np.array(edge_xs), \n\
            y=np.array(edge_ys), \n\
            mode='lines', \n\
            name='hull', \n\
            hoverinfo='none' \n\
        ) \n\
        return hull_scatter \n\
 \n\
    def _create_points_scatter(self, points_data, em1, left_color, right_color): \n\
        colors, auids, urls, c_name, xs, ys = [], [], [], [], [], [] \n\
 \n\
        # First grab points from the points data \n\
        for compound in points_data: \n\
            # Add AUID and url \n\
            urls.append(compound['url_entry_page']) \n\
            auids.append(compound['auid']) \n\
            # Add compound name \n\
            c_name.append(compound['compound']) \n\
            # Set y: enthalpy_formation_atom \n\
            y = compound['enthalpy_formation_atom'] \n\
            # Set x \n\
            frac = compound['fractional_compound'] \n\
            # If 1 element then figure out if first or second element \n\
            if compound['nspecies'] == 1: \n\
                x = 0.0 if frac == em1 else 1.0 \n\
            # o.w. mix of the 2 elements \n\
            else: \n\
                # Example search: 'Pd0.4Pt0.6' -> '0.6' \n\
                x = float(re.search(r'(0.\\d*)$', frac).group(0)) \n\
            # Add color \n\
            new_color = [(x * right_color[i]) + ((1-x) * left_color[i]) for i in range(3)] \n\
            colors.append(new_color) \n\
            # Add point \n\
            xs.append(x) \n\
            ys.append(y) \n\
 \n\
        # Create scatter \n\
        points_scatter = go.Scatter( \n\
        x=np.array(xs), \n\
        y=np.array(ys), \n\
        mode='markers', \n\
        name='enthalpy', \n\
        hoverinfo='text', \n\
        text=[\"\"\"<a href=\"{}/\">{}</a>\"\"\".format(urls[i], c_name[i]) for i in range(len(urls))], \n\
        hoverlabel=dict(bgcolor='black'), \n\
        marker=dict( \n\
            color=['rgb({}, {}, {})'.format(c[0], c[1], c[2]) for c in colors], \n\
            line=dict( \n\
                color='rgb(0, 0, 0)', \n\
                width=1 \n\
                ) \n\
            ), \n\
        ) \n\
 \n\
        return points_scatter \n\
 \n\
    def _create_figure(self, hull_scatter, points_scatter, em1, em2): \n\
        data = [hull_scatter, points_scatter] \n\
        layout = go.Layout(showlegend=False, \n\
                           hovermode='closest', \n\
                           xaxis=dict( \n\
                                title='{}-{}'.format(em1, em2), \n\
                                autorange=True, \n\
                                showgrid=True, \n\
                                zeroline=False, \n\
                                showline=True, \n\
                                showticklabels=True, \n\
                                titlefont=dict( \n\
                                    size=18, \n\
                                ) \n\
                            ), \n\
                            yaxis=dict( \n\
                                title='eV/Atom', \n\
                                autorange=True, \n\
                                showgrid=True, \n\
                                zeroline=False, \n\
                                showline=True, \n\
                                showticklabels=True, \n\
                                titlefont=dict( \n\
                                    size=18, \n\
                                ) \n\
                            )) \n\
        fig = go.Figure(data=data, layout=layout) \n\
        return fig \n\
     \n\
     \n\
# For plotting 3 species alloys (Ternary) \n\
class _Three_Species_Plotter: \n\
    def plot_CHull(self, hull, em1, em2, em3): \n\
        self.em1, self.em2, self.em3 = em1, em2, em3 \n\
        # ----- Create scatter for edges ----- \n\
        # Grab compounds that make up points for edges \n\
        compounds = [dic['vertices_position'] for dic in hull['facets_data']['3-nary:{}-{}-{}'.format(em1, em2, em3)]] \n\
        # Create scatter for edges \n\
        hull_scatters = self._create_edges_scatter(compounds) \n\
 \n\
        # ----- Create scatter for points ----- \n\
        points_scatter = self._create_points_scatter(hull['points_data']) \n\
 \n\
        # ----- Plot ----- \n\
        fig = self._create_figure(hull_scatters, points_scatter) \n\
        iplot(fig) \n\
 \n\
    def _create_edges_scatter(self, compounds): \n\
        hull_scatters = [] \n\
        points_a, points_b, points_c = [], [], [] \n\
        for vertices in compounds: \n\
            point1, point2, point3 = vertices[0], vertices[1], vertices[2] \n\
            point1[2] = 1.0 - (point1[0] + point1[1]) \n\
            point2[2] = 1.0 - (point2[0] + point2[1]) \n\
            point3[2] = 1.0 - (point3[0] + point3[1]) \n\
            # Important note: As and Bs are switched in the json compared to ternary plot points in Plotly \n\
            hull_scatter = { \n\
                'type': 'scatterternary', \n\
                'mode': 'lines', \n\
                'name': 'hull', \n\
                'hoverinfo': 'none', \n\
                'marker': dict(color='rgba(150, 150, 150, 0.4)', size=0.5), \n\
                'a': np.array([point1[1], point2[1], point3[1]]), \n\
                'b': np.array([point1[0], point2[0], point3[0]]), \n\
                'c': np.array([point1[2], point2[2], point3[2]]) \n\
            } \n\
            hull_scatters.append(hull_scatter) \n\
         \n\
        return hull_scatters \n\
     \n\
    def _create_points_scatter(self, points_data): \n\
        colors, auids, urls, c_name, As, Bs, Cs = [], [], [], [], [], [], [] \n\
 \n\
        # First grab points from the points data \n\
        for compound in points_data: \n\
            # Add AUID and url \n\
            urls.append(compound['url_entry_page']) \n\
            auids.append(compound['auid']) \n\
            # Add compound name \n\
            c_name.append(compound['compound']) \n\
            comp = compound['fractional_compound'] \n\
            if compound['nspecies'] == 1: \n\
                if comp == self.em1: vals = [1, 0, 0] \n\
                elif comp ==  self.em2: vals = [0, 1, 0] \n\
                else: vals = [0, 0, 1] \n\
            else: \n\
                em_se = [re.search('{}(0.\\d*)'.format(em), comp) for em in [self.em1, self.em2, self.em3]] \n\
                vals = [float(v.group(1)) if v else 0.0 for v in em_se] \n\
            As.append(vals[1]) \n\
            Bs.append(vals[0]) \n\
            Cs.append(vals[2]) \n\
            # Add color \n\
            r, g, b = [1,0,0], [0,1,0], [0,0,1] \n\
            new_color = [(As[-1] * b[i]) + (Bs[-1] * r[i]) + (Cs[-1] * g[i]) for i in range(3)] \n\
            colors.append(new_color) \n\
         \n\
        points_scatter = { \n\
                'type': 'scatterternary', \n\
                'mode': 'markers', \n\
                'name': 'points', \n\
                'hoverinfo': 'text', \n\
                'a': np.array(As), \n\
                'b': np.array(Bs), \n\
                'c': np.array(Cs), \n\
                'text': [\"\"\"<a href=\"{}/\">{}</a>\"\"\".format(urls[i], c_name[i]) for i in range(len(urls))], \n\
                'hoverlabel': dict(bgcolor='black'), \n\
                'marker': dict( \n\
                size = 7, \n\
                color=['rgb({}, {}, {})'.format(c[0], c[1], c[2]) for c in colors], \n\
                line=dict( \n\
                    color='rgb(0, 0, 0)', \n\
                    width=1 \n\
                    ) \n\
                ) \n\
            } \n\
 \n\
        return points_scatter \n\
 \n\
    def _create_figure(self, hull_scatters, points_scatter): \n\
        data = hull_scatters \n\
        data.append(points_scatter) \n\
         \n\
        layout = { \n\
            'ternary':{ \n\
                'sum': 1.0, \n\
                'aaxis': dict(title=self.em2, showgrid=False, showticklabels=False, titlefont=dict(color='blue')), \n\
                'baxis': dict(title=self.em1, showgrid=False, showticklabels=False, titlefont=dict(color='red')), \n\
                'caxis': dict(title=self.em3, showgrid=False, showticklabels=False, titlefont=dict(color='green')) \n\
            }, \n\
            'showlegend': False, \n\
            'hovermode': 'closest', \n\
        } \n\
         \n\
        fig = {'data': data, 'layout': layout} \n\
        return fig \n\
";
